/* -*- Mode: C++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 8 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Telemetry uses this interface on the @mozilla.org/js/xpc/XPConnect;1 service
 * to extract JS engine stats.
 */
[scriptable, uuid(5a6ea52b-4e23-402f-93e3-59f29b2f1a88)]
interface nsIJSEngineTelemetryStats : nsISupports
{
  /**
   * The value returned by this attribute is included as the 'js' property of
   * the telemetry ping JSON blob.
   */
  [implicit_jscontext]
  readonly attribute jsval telemetryValue;
};
